#version 330 core
//Light Shader
//Fratment SHADER

struct Material {
	vec3 ambientColor;			//Ambiente Farbe
	bool diffuseHasTexture;		//Gibt an, ob es eine Diffus-Textur gibt
	vec3 diffuseColor;			//Falls keine Textur: Diffus-Farbe
	sampler2D texture;			//Ansonsten: Textur
	vec3 specularColor;			//Specular-Farbe
	float shininess;			//Specular-Shininess
	float opacity;				//Undurchsichtbarkeit
};

struct PointLight {
	vec3 position;
	vec3 intensity;
	float constantAttenuation;
	float linearAttenuation;
	float quadraticAttenuation;
};
struct DirectionalLight {
	vec3 intensity;
	vec3 direction;
};
struct ShadowMapData {
	samplerCube shadowTexture;
	vec3 spillPosition;
};
#define NR_MAX_POINTLIGHTS 5
#define NR_MAX_DIRECTIONALLIGHTS 5
#define NR_MAX_SHADOWMAPS 2
#define FAR_PLANE 80.0f

//In: UV
in vec2 fUv;
//In: Normale
in vec3 fNormal;
//In: Richtung zum Auge
in vec3 fEye;
//In: Position
in vec3 fPosition;

//Out: Farbe
layout (location = 0) out vec4 fragColor;
//Out: Glow-Farbe
layout (location = 1) out vec4 brightColor;

//Uniform: Material des Objektes
uniform Material material;
//Uniform: Lichter
uniform PointLight pointLights[NR_MAX_POINTLIGHTS];
uniform DirectionalLight directionalLights[NR_MAX_DIRECTIONALLIGHTS];
//Uniform: Lichter-Anzahl
uniform int pointLightCount;
uniform int directionalLightCount;
//Uniform: Model-Matrix
uniform mat4 model;
//Uniform: Die verwendeten ShadowMaps
uniform ShadowMapData shadowMaps[NR_MAX_SHADOWMAPS];
//Uniform: Welche ShadowMaps verwendet werden
uniform bool useShadowMap1;
uniform bool useShadowMap2;
//Uniform: Kameraposition
uniform vec3 eyePos;
//Uniform: Transparenz ein/ausschalten (0 => aus / 1 => ein)
uniform int transparencyMode;

vec3 calcPointLight(PointLight light) {
	vec3 l = light.position - fPosition;
	float distance = length(l);
	l = normalize(l);
	vec3 normal = normalize(fNormal);
	float A = light.constantAttenuation + distance*light.linearAttenuation + pow(distance, 2)*light.quadraticAttenuation;
	vec3 intensity = light.intensity / A;
	float nl = dot(normal, l);
	vec3 diffuse = vec3(0, 0, 0);
	if (nl > 0) {
		if (material.diffuseHasTexture) {
			diffuse = (texture(material.texture, fUv).rgb) * intensity * nl;
		} else {
			diffuse = (material.diffuseColor.rgb) * intensity  * nl;
		}	
	}
	vec3 specular = vec3(0, 0, 0);
	if (nl > 0) {
		vec3 h = normalize(l + fEye);
		float nhp =  pow(dot(normal, h), material.shininess);
		if (nhp > 0) {
			specular = material.specularColor.rgb * intensity * nhp;
		}
	}
	return diffuse + specular;
}

vec3 calcDirectionalLight(DirectionalLight light) {
	vec3 l = normalize(-light.direction);
	vec3 intensity = light.intensity;
	vec3 normal = normalize(fNormal);
	float nl = dot(normal, l);
	vec3 diffuse = vec3(0, 0, 0);
	if (nl > 0) {
		if (material.diffuseHasTexture) {
			diffuse = (texture(material.texture, fUv).rgb) * intensity * nl;
		} else {
			diffuse = (material.diffuseColor.rgb) * intensity  * nl;
		}	
	}
	vec3 specular = vec3(0, 0, 0);
	if (nl > 0) {
		vec3 h = normalize(l + fEye);
		float nhp =  pow(dot(normal, h), material.shininess);
		if (nhp > 0) {
			specular = material.specularColor.rgb * intensity * nhp;
		}
	}
	return diffuse + specular;
}

#define SAMPLECOUNT 20

vec3 sampleOffsetDirections[SAMPLECOUNT] = vec3[]
(
   vec3( 1,  1,  1), vec3( 1, -1,  1), vec3(-1, -1,  1), vec3(-1,  1,  1), 
   vec3( 1,  1, -1), vec3( 1, -1, -1), vec3(-1, -1, -1), vec3(-1,  1, -1),
   vec3( 1,  1,  0), vec3( 1, -1,  0), vec3(-1, -1,  0), vec3(-1,  1,  0),
   vec3( 1,  0,  1), vec3(-1,  0,  1), vec3( 1,  0, -1), vec3(-1,  0, -1),
   vec3( 0,  1,  1), vec3( 0, -1,  1), vec3( 0, -1, -1), vec3( 0,  1, -1)
);   

float updateLightMultiplier(float lightmult, int mapIndex) {
	
	vec3 fragToLight = fPosition - shadowMaps[mapIndex].spillPosition;
	float currentDepth = length(fragToLight);

	//PCF according to http://learnopengl.com/#!Advanced-Lighting/Shadows/Point-Shadows
	float shadow = 0.0;
	float viewDistance = distance(eyePos, fPosition);
	float diskRadius = 0.02;

	for (int i = 0; i < SAMPLECOUNT; ++i) {
		float closestDepth = 0;
		if (mapIndex == 0) {
			closestDepth = texture(shadowMaps[0].shadowTexture, fragToLight + sampleOffsetDirections[i] * diskRadius).x;
		} else if (mapIndex == 1) {
			closestDepth = texture(shadowMaps[1].shadowTexture, fragToLight + sampleOffsetDirections[i] * diskRadius).x;
		}
		closestDepth *= FAR_PLANE;
		if (currentDepth - 0.05 >= closestDepth) {
			shadow += 1.0;
		}
	}
	shadow /= float(SAMPLECOUNT);


	if (shadow != 0) {
		float newlightmult = 1-shadow;
		float shadowval = min(1, currentDepth/10.f);
		newlightmult = max(0.3, shadowval + (newlightmult*(1-shadowval)));
		if (newlightmult < lightmult) {
			return newlightmult;
		}
	}
	return lightmult;
}

void main() {
	vec3 res = material.ambientColor;
	float lightmult = 1;
	for (int i = 0; i < pointLightCount && i < NR_MAX_POINTLIGHTS; i++) {
		res += calcPointLight(pointLights[i]);
	}
	for (int i = 0; i < directionalLightCount && i < NR_MAX_DIRECTIONALLIGHTS; i++) {
		res += calcDirectionalLight(directionalLights[i]);
	}
	if (useShadowMap1) {
		lightmult = updateLightMultiplier(lightmult, 0);
	}
	if (useShadowMap2) {
		lightmult = updateLightMultiplier(lightmult, 1);
	}
	res *= lightmult;

	if(transparencyMode == 1)
		fragColor = vec4(res, material.opacity);
	else
		fragColor = vec4(res, 1);

	float brightness = dot(fragColor.rgb, vec3(0.2126, 0.7152, 0.0722));
    if(brightness > 0.8)
        brightColor = vec4(fragColor.rgb, 1.0);
	else
		brightColor = vec4(0.0, 0.0, 0.0, 1.0);
}